<?php

namespace App\Services;

use PDO;

/**
 * Service d'authentification utilisateur.
 * Gère les connexions, déconnexions et vérifie l'état de l'authentification.
 */
class AuthService
{
    private $db;         // Instance PDO pour l'accès à la base de données
    private $logService; // Service de journalisation

    /**
     * Constructeur du service d'authentification.
     *
     * @param PDO $db L'instance PDO de la base de données.
     */
    public function __construct(PDO $db)
    {
        $this->db = $db;
        // Le chemin du fichier de log doit être défini dans config/app.php
        $this->logService = new LogService(APP_LOG_PATH);
    }

    /**
     * Tente de connecter un utilisateur.
     *
     * @param string $username Le nom d'utilisateur.
     * @param string $password Le mot de passe en clair.
     * @param string $ipAddress L'adresse IP de la requête de connexion.
     * @return bool Vrai si la connexion est réussie, faux sinon.
     */
    public function login($username, $password, $ipAddress)
    {
        $stmt = $this->db->prepare("SELECT id, username, password, role FROM users WHERE username = ?");
        $stmt->execute([$username]);
        $user = $stmt->fetch();

        if ($user && password_verify($password, $user['password'])) {
            // Connexion réussie : enregistre les informations de l session
            $_SESSION['user_id'] = $user['id'];
            $_SESSION['username'] = $user['username'];
            $_SESSION['role'] = $user['role'];

            // Log de l'action de connexion réussie
            $this->logService->log('info', "Connexion réussie pour l'utilisateur '{$username}'.", $user['id'], $ipAddress);
            return true;
        } else {
            // Connexion échouée : log de la tentative
            $this->logService->log('warning', "Tentative de connexion échouée pour l'utilisateur '{$username}'.", null, $ipAddress);
            return false;
        }
    }

    /**
     * Déconnecte l'utilisateur actuellement connecté.
     *
     * @param string $ipAddress L'adresse IP de la requête de déconnexion.
     * @return bool Vrai si la déconnexion est effectuée.
     */
    public function logout($ipAddress)
    {
        $userId = $_SESSION['user_id'] ?? 'inconnu';
        $username = $_SESSION['username'] ?? 'inconnu';

        // Détruit toutes les données de session
        session_destroy();
        $_SESSION = array(); // Vider le tableau $_SESSION

        // Log de l'action de déconnexion
        $this->logService->log('info', "Déconnexion de l'utilisateur '{$username}'.", $userId, $ipAddress);
        return true;
    }

    /**
     * Vérifie si un utilisateur est actuellement connecté.
     *
     * @return bool Vrai si l'utilisateur est connecté, faux sinon.
     */
    public function isLoggedIn()
    {
        return isset($_SESSION['user_id']);
    }

    /**
     * Récupère le rôle de l'utilisateur connecté.
     *
     * @return string|null Le rôle de l'utilisateur ('admin' ou 'user'), ou null si non connecté.
     */
    public function getUserRole()
    {
        return $_SESSION['role'] ?? null;
    }

    /**
     * Récupère l'ID de l'utilisateur connecté.
     *
     * @return int|null L'ID de l'utilisateur, ou null si non connecté.
     */
    public function getUserId()
    {
        return $_SESSION['user_id'] ?? null;
    }

    /**
     * Récupère le nom d'utilisateur de l'utilisateur connecté.
     *
     * @return string|null Le nom d'utilisateur, ou null si non connecté.
     */
    public function getUsername()
    {
        return $_SESSION['username'] ?? null;
    }
}
